#![allow(dead_code)]

mod util;
mod template; use template::Type;
mod data; pub use data::*;

pub enum Szun {
    Null,
    Boolean(bool),
    Natural(u64),
    Integer(i64),
    Decimal(f64),
    Block(Vec<u8>),
    String(String),
    Set(Vec<Szun>),
    Array(Vec<Szun>),
    List(Vec<Szun>),
    Sparse(Vec<Szun>),
    Map(Vec<Szun>),
    //Tree(Vec<Szun>),
    //Graph(Vec<Szun>),
    Enum(Vec<Szun>),
    Selection(Vec<Szun>),
    Record(Vec<Szun>),
    Schema(Vec<Szun>),
}
impl Szun {
    fn encode_parts(&self) -> (Vec<u32>, Vec<u8>)
    {
        let mut tags = Vec::<u32>::with_capacity(1);
        let mut encoded = Vec::<u8>::with_capacity(64);

        match self {
            Self::Null => {
                tags.push(Null::tag());
            }
            
            Self::Boolean(_data) => {
                tags.push(Boolean::tag());
            }
            
            Self::Natural(data) => {
                tags.push(Natural::tag());
                encoded.append(&mut util::pack_natural(*data));
            }
            
            Self::Integer(data) => {
                tags.push(Integer::tag());
                encoded.append(&mut &mut util::pack_integer(*data));
            }

            _ => {
                tags.push(0);
            }
            
            /*Self::Decimal(_data) => {
                tag = Decimal::tag();
            }
            
            Self::Block(_data) => {
                tag = Block::tag();
            }
            
            Self::String(_data) => {
                tag = String::tag();
            }
            
            Self::Set(_data) => {
                tag = Set::tag();
            }
            
            Self::Array(_data) => {
                tag = Array::tag();
            }
            
            Self::List(_data) => {
                tag = List::tag();
            }
            
            Self::Sparse(_data) => {
                tag = Sparse::tag();
            }
            
            Self::Map(_data) => {
                tag = Map::tag();
            }
            
            Self::Enum(_data) => {
                tag = Enum::tag();
            }
            
            Self::Selection(_data) => {
                tag = Selection::tag();
            }
            
            Self::Record(_data) => {
                tag = Record::tag();
            }

            Self::Schema(_data) => {
                tag = Schema::tag();
            }*/
        }

        return (tags, encoded);
    }

    pub fn encode_data(&self) -> Vec<u8>
    {
        let (_, encoded) = self.encode_parts();
        return encoded;
    }

    pub fn encode(&self) -> Vec<u8>
    {
        let (tags, data) = self.encode_parts();
        let mut prefix = Vec::<u8>::with_capacity(16);

        for tag in tags {
            prefix.append(&mut util::pack_natural(tag as u64));
        }

        let mut encoded = Vec::<u8>::with_capacity(prefix.len() + data.len());
        for b in prefix { encoded.push(b); }
        for b in data { encoded.push(b); }

        return encoded;
    }
}
